/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.modules.refactoring.sourceactions;

import javax.swing.text.Document;
import org.netbeans.modules.editor.indent.api.Reformat;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;
import org.openide.cookies.EditorCookie;

/**
 * Action to Format all Files in a Folder with the Indentation API.
 * @author lenzweber
 */
public final class FormatMultipleAction extends CookieAction {

    protected void performAction(Node[] activatedNodes) {
        DataFolder df = activatedNodes[0].getLookup().lookup(DataFolder.class);
        recurse(df);
    }

    /**
     * Either recurses deeper into the dataObject-Tree or tries to format the underlying Document.
     * @param dataObject Object to Recurse on.
     */
    private void recurse(DataObject dataObject) {
        if (dataObject instanceof DataFolder) {
            for (DataObject child : ((DataFolder) dataObject).getChildren()) {
                recurse(child);
            }
        } else {
            try {
                EditorCookie editorCookie = dataObject.getLookup().lookup(EditorCookie.class);
                boolean modified = editorCookie.isModified();
                Document doc = editorCookie.openDocument();
                Reformat reformat = Reformat.get(doc);
                reformat.lock();
                try {
                    reformat.reformat(0, doc.getLength());
                } finally {
                    reformat.unlock();
                }
                if (!modified){
                    editorCookie.saveDocument();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    protected int mode() {
        return CookieAction.MODE_ALL;
    }

    public String getName() {
        return NbBundle.getMessage(FormatMultipleAction.class, "CTL_FormatMultipleAction");
    }

    protected Class[] cookieClasses() {
        return new Class[]{DataFolder.class};

    }

    @Override
    protected void initialize() {
        super.initialize();
        // see org.openide.util.actions.SystemAction.iconResource() Javadoc for more details
        putValue("noIconInMenu", Boolean.TRUE);
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    protected boolean asynchronous() {
        return false;
    }
}

