/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.modules.refactoring.sourceactions;

import org.netbeans.modules.java.editor.imports.JavaFixAllImports;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.cookies.EditorCookie;
import org.openide.util.actions.NodeAction;

public final class JavaFixImportsAction extends NodeAction {

    private static final String MIME = "text/x-java";

    protected void performAction(Node[] activatedNodes) {
        DataFolder df = activatedNodes[0].getLookup().lookup(DataFolder.class);
        recurse(df);
    }

    private void recurse(DataObject dataObject) {
        if (dataObject instanceof DataFolder) {
            for (DataObject child : ((DataFolder) dataObject).getChildren()) {
                recurse(child);
            }
        } else if (dataObject.getPrimaryFile().getMIMEType().equals(MIME)) {
            try {
                EditorCookie editorCookie = dataObject.getLookup().lookup(EditorCookie.class);
                boolean modified = editorCookie.isModified();
                editorCookie.openDocument();
                //works only if Document opened
                JavaFixAllImports.getDefault().fixAllImports(dataObject.getPrimaryFile());
                if (!modified) {
                    editorCookie.saveDocument();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

    }

    public String getName() {
        return NbBundle.getMessage(JavaFixImportsAction.class, "CTL_JavaFixImportsAction");
    }

    @Override
    protected void initialize() {
        super.initialize();
        // see org.openide.util.actions.SystemAction.iconResource() Javadoc for more details
        putValue("noIconInMenu", Boolean.TRUE);
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    protected boolean asynchronous() {
        return false;
    }

    @Override
    protected boolean enable(Node[] activatedNodes) {
        DataFolder df = activatedNodes[0].getLookup().lookup(DataFolder.class);

        return folderContainsClass(df, MIME);
    }

    private boolean folderContainsClass(DataObject dataObject, String mimeType) {
        if (dataObject == null) {
            return false;
        } else if (dataObject instanceof DataFolder) {
            for (DataObject child : ((DataFolder) dataObject).getChildren()) {
                if (folderContainsClass(child, mimeType)) {
                    return true;
                }
            }
        } else if (dataObject.getPrimaryFile().getMIMEType().equals(mimeType)) {
            return true;
        }
        return false;
    }
}

