/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.modules.refactoring.sourceactions;

import java.io.IOException;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.openide.awt.Actions;
import org.openide.cookies.InstanceCookie;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.filesystems.Repository;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;

/**
 * Context Menu Item, provides a SubMenu with Entries from 
 * "Actions/Source/Folder" in the Layer file.
 * @author lenzweber
 */
public final class SourcePopupAction extends CookieAction {

    protected void performAction(Node[] activatedNodes) {
        //do nothing.
    }

    protected int mode() {
        return CookieAction.MODE_EXACTLY_ONE;
    }

    public String getName() {
        return NbBundle.getMessage(SourcePopupAction.class, "CTL_SourcePopupAction");
    }

    protected Class[] cookieClasses() {
        return new Class[]{DataObject.class};
    }

    @Override
    protected void initialize() {
        super.initialize();
        // see org.openide.util.actions.SystemAction.iconResource() Javadoc for more details
        putValue("noIconInMenu", Boolean.TRUE);
    }

    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }

    @Override
    protected boolean asynchronous() {
        return false;
    }

    @Override
    public JMenuItem getPopupPresenter() {
        return subMenu;
    }
    /**
     * Submenu. Lazy creation from DefaultFileSystem inspired by
     * org.netbeans.modules.refactoring.spi.impl.RefactoringSubMenuAction
     */
    private JMenu subMenu = new JMenu(getName()) {

        private boolean initialized = false;

        /** Gets popup menu. Overrides superclass. Adds lazy menu items creation. */
        @Override
        public JPopupMenu getPopupMenu() {
            if (!initialized) {
                synchronized (this) {
                    if (!initialized) {
                        createMenuItems();
                        initialized = true;
                    }
                }
            }
            return super.getPopupMenu();
        }

        /** Creates items when actually needed. */
        private void createMenuItems() {
            removeAll();
            FileSystem dfs = Repository.getDefault().getDefaultFileSystem();
            FileObject fo = dfs.findResource("Actions/Source/Folder");
            DataFolder df = DataFolder.findFolder(fo);

            if (df != null) {
                for (DataObject dataObject : df.getChildren()) {
                    InstanceCookie ic = (InstanceCookie) dataObject.getCookie(InstanceCookie.class);
                    if (ic == null) {
                        continue;
                    }
                    try {
                        if (!Action.class.isAssignableFrom(ic.instanceClass())) {
                            continue;
                        }
                        Action instance = (Action) ic.instanceCreate();
                        JMenuItem mi = new JMenuItem();
                        Actions.connect(mi, instance);
                        this.add(mi);
                    } catch (IOException e) {
                        // ignore
                        e.printStackTrace();
                        continue;
                    } catch (ClassNotFoundException e) {
                        // ignore
                        e.printStackTrace();
                        continue;
                    }
                }
            }
        }
    };
}

